# PowerNet Transfer Protocol

Powernet transfer protocol is a remote procedure call for creating spooky action at a distance.

It can discover a device and all of its properties and use those properties as 
variables in a python program. 

It attemtps to gurarantee message delivery as much as possible through use of keep-alive signals, crc, retransmission, & magic.

Usage is simple:

    import powernet
    
    def on_connect(domain, computer):
        # Do something here when new computer appears   
        pass

    def on_disconnect(domain):
        # Do something here to handle lost connection 
        pass

    if __name__ == '__main__':
        powernet.start(
            new_connection = on_connect, 
            lost_connection = on_disconnect, 
            runtime_globals = globals()
        )
 

### Performance 

149 kilometers / second on USB 1.1


### Remote Computers

Remote computers are accessible in the namespace **pntp**.
    
Type

    pntp

Into the interactive shell to see all the discoverable computers.

    In [1]: pntp
    Out[1]: 
      my_computer,
      computer_b

Type 

    pntp.my_computer
    
To access remote computer variables.

    In [2]: pntp.my_computer                                         
    Out[2]: 
    properties:
        bool_property = 0 (volatile bool)
        float_property = 1.0 (volatile float)
        int_property = 1 (volatile int64_t) [min: -20, max: 20]
        string_property = 'Hello World' (volatile String[1024])
        buffer_property = RemoteBytearray(b'{|}') (volatile char*[64])
        callable_function(input: i64)
    interrupts:
        event() = None (event handler)
    network:
        domain: 'my_computer'  (nonvolatile String[32])
        channel: 0
    system:
        info: ...
        reboot()
        dfu()


### Naming a Computer

You can assign a computer a unique ID with 

    pntp.my_computer.network.domain = "some_name"

The computer will reconnect and appear under its new name. 

The name will be remembered, unless the computer's memory is erased or it is assigned a new name.


### Interrupts

You may enable interrupts by creating an event handler function

    def interruptHandler(): 
        pass
        
And assign it to any interrupts the computer supports 

    pntp.[domain].interrupts.[interrupt_name] = interruptHandler
    
No arguments are passed to interruptHandler, and it should not return anything.

You can type
    
    pntp.[domain].interrupts
    
To see what interrupts are available, if any. 


### Computer Firmware Update

A computer can be put into DFU mode with 

    pntp.[domain].dfu()
    
It will stay in DFU mode unless reset or flashed with new firmware.


### Command Line Arguments

**--verbose** print debug information

**--noshell** disable interactive shell

**--tty** allow program discover tty/acm ports

**--tty_nat** forward tty/acm ports over tcp

**--ping_time** minimum time before attempting packet retransmission (s)

**--pong_time** maximum time before assuming connection is dead (s)

**--dns_port** dns is accessible on this port. default: 42000, 0: disabled

**--dns_domains []** csv list of domains to look for

**--dns_channels []** csv list of channels to look for

**--dns_vendors []** csv list of vendors to look for

**--dns_products []** csv list of products to look for

**--dns_versions []** csv list of versions to look for